
# q-expansion of newform 175.5.d.c, downloaded from the LMFDB on 06 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 175
weight = 5
poly_data = [1, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[127, [1, 0]], [101, [-1, 0]]]
aps_data = [[0, 0], [0, 17], [0, 0], [0, 49], [-73, 0], [0, -23], [0, 263], [0, 0], [0, 0], [1153, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, -3457], [0, 0], [0, 0], [0, 0], [0, 0], [-10078, 0], [0, 9502], [-12167, 0], [0, 6382], [0, 0], [0, 3383], [0, 0], [0, -18383], [0, 0], [14353, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [-24242, 0], [-45433, 0], [0, -31342], [0, 0], [0, 17663], [0, 11017], [16558, 0], [0, 0], [47447, 0], [0, 0], [0, 0], [0, 0], [-77593, 0], [0, -61343], [0, 49823], [0, 0], [0, 0], [-43367, 0], [0, 0], [0, 0], [0, 111938], [0, 0], [0, 0], [0, 0], [0, 0], [119807, 0], [0, -152303], [0, 171337], [0, 135263], [0, 0], [0, 147097], [0, 0], [138482, 0], [0, 0], [0, 0], [0, 0], [0, 229897], [-76322, 0], [0, -116497], [0, 0], [35278, 0], [0, 29182], [-249407, 0], [0, -163897], [-316273, 0], [0, 0], [0, 0], [-76753, 0], [356087, 0], [0, -193538], [0, 0], [0, 0], [-264287, 0], [0, 0], [0, 0], [0, 0], [0, 322463], [0, 0], [0, 0], [-470713, 0], [31513, 0], [0, 313297], [0, 0], [0, 0], [0, 440782], [2927, 0], [0, 0], [0, 0], [0, -129938], [-566882, 0], [-638158, 0], [0, -665017], [0, 507698], [0, 320137], [613273, 0], [0, 0], [0, -82417], [0, 0], [0, 0], [0, 0], [100487, 0], [96002, 0], [0, -713183], [0, 111458], [0, 0], [-777527, 0], [0, 0], [0, 0], [0, 750023], [0, 0], [0, 0], [-884833, 0], [-1002527, 0], [0, 0], [0, 976418], [0, -51143], [-749207, 0], [0, 0], [648887, 0], [0, 0], [0, 0], [0, 0], [0, -1179623], [0, -1037137], [0, 1073543], [1299553, 0], [0, 0], [1234367, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 1447822], [0, -970462], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, -1329502], [0, 0], [1155842, 0], [695113, 0], [0, 0], [0, 1589303], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 0], [0, 675937], [-1442878, 0], [0, 737783]]
